@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2014-2023 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2014-2025 Cisco and/or its affiliates. All rights reserved.
 ** Copyright (C) 2010-2013 Sourcefire, Inc.
 ** Author: Michael R. Altizer <mialtize@cisco.com>
 **
@@ -32,7 +32,7 @@ extern "C" {
 #include <unistd.h>
 
 // Comprehensive version number covering all elements of this header
-#define DAQ_COMMON_API_VERSION  0x00030005
+#define DAQ_COMMON_API_VERSION  0x00030010
 
 #ifndef DAQ_SO_PUBLIC
 #  ifdef HAVE_VISIBILITY
@@ -302,6 +302,7 @@ typedef struct _daq_flow_stats
     uint16_t vlan_tag;
     uint8_t protocol;
     uint8_t flags;
+    uint64_t drop_reason;
 } DAQ_FlowStats_t;
 
 /* Packet verdicts passed to daq_msg_finalize(). */
@@ -351,6 +352,7 @@ typedef struct _daq_stats
     uint64_t packets_received;          /* Packets received by this instance */
     uint64_t packets_filtered;          /* Packets filtered by this instance's BPF */
     uint64_t packets_injected;          /* Packets injected by this instance */
+    uint64_t packets_outstanding;       /* Packets outstanding in this instance */
     uint64_t verdicts[MAX_DAQ_VERDICT]; /* Counters of packets handled per-verdict. */
 } DAQ_Stats_t;
 
@@ -392,6 +394,7 @@ typedef struct _daq_msg_pool_info
 #define DAQ_CAPA_DECODE_MPLS    0x00020000   /* decodes and tracks flows within MPLS. */
 #define DAQ_CAPA_DECODE_VXLAN   0x00040000   /* decodes and tracks flows within VXLAN. */
 #define DAQ_CAPA_DECODE_GENEVE  0x00080000   /* decodes and tracks flows within Geneve. */
+#define DAQ_CAPA_INJECT_DROP    0x00100000   /* can call ioctl() for DIOCTL_SET_INJECT_DROP */
 
 /*
  * DAQ I/O Controls (DIOCTLs)
@@ -412,6 +415,9 @@ typedef enum
     DIOCTL_DIRECT_INJECT_PAYLOAD,
     DIOCTL_DIRECT_INJECT_RESET,
     DIOCTL_GET_PRIV_DATA_LEN,
+    DIOCTL_GET_CPU_PROFILE_DATA,
+    DIOCTL_GET_SNORT_LATENCY_DATA,
+    DIOCTL_SET_INJECT_DROP,
     LAST_BUILTIN_DIOCTL_CMD = 1024,     /* End of reserved space for "official" DAQ ioctl commands.
                                            Any externally defined ioctl commands should be larger than this. */
     MAX_DIOCTL_CMD = UINT16_MAX
@@ -628,6 +634,46 @@ typedef struct
     uint16_t priv_data_len;  // [out] Length of priv data.
 } DIOCTL_GetPrivDataLen;
 
+/*
+ * Command: DIOCTL_GET_CPU_PROFILE_DATA 
+ * Description: Get CPU Profile Data
+ * Argument: DIOCTL_GetCpuProfileData 
+ */
+typedef struct
+{
+    float cpu_usage_percent_30s;   /* [out] cpu profile data for the last  30 seconds  */
+    float cpu_usage_percent_120s;  /* [out] cpu profile data for the last 120 seconds */
+    float cpu_usage_percent_300s;  /* [out] cpu profile data for the last 300 seconds */
+} DIOCTL_GetCpuProfileData;
+
+
+/* Command: DIOCTL_GET_SNORT_LATENCY_DATA
+* Description: Get Snort Latency Data
+* Argument: DIOCTL_GetSnortLatencyData
+*/
+
+typedef enum
+{
+    DAQ_SNORT_LATENCY_PROTO_TCP,     /* TCP flow Snort processing time */
+    DAQ_SNORT_LATENCY_PROTO_UDP,     /* UDP flow Snort processing time */
+    DAQ_SNORT_LATENCY_PROTO_OTHERS,  /* Non TCP, UDP Snort processing time */
+    DAQ_SNORT_LATENCY_PROTO_MAX
+} DAQ_snort_latency_proto_t;
+
+typedef struct _daq_snort_latency_data
+{
+    uint64_t max_pkt_time;             /* Max packet snort processing latency seen in last 5 minutes */
+    uint64_t snort_up_max_pkt_time;    /* Max packet snort processing latency seen from snort up */
+    uint64_t pkt_count;                /* Total packets pulled by snort for processing in last 5 minutes */
+    uint64_t sum_time;                 /* Total latency of all packets processed by snort in last 5 minutes */
+    uint64_t conn_meta_null_counters;  /* Number of times packet conn meta was null in last 5 minutes */
+} DAQ_snort_latency_data_t;
+
+typedef struct {
+    /* Array of Snort processing time based on protocol */
+    DAQ_snort_latency_data_t snort_latency_data[DAQ_SNORT_LATENCY_PROTO_MAX];
+} DIOCTL_GetSnortLatencyData;
+
 #ifdef __cplusplus
 }
 #endif
