@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2018-2023 Cisco and/or its affiliates. All rights reserved.
+** Copyright (C) 2018-2025 Cisco and/or its affiliates. All rights reserved.
 ** Author: Michael R. Altizer <mialtize@cisco.com>
 **
 ** This program is free software; you can redistribute it and/or modify
@@ -30,6 +30,7 @@ extern "C" {
 
 /* Relevant ethertypes lifted from Linux's if_ether.h since there doesn't seem to be a reliable
     cross-platform way of obtaining all of them. */
+#define ETYPE_MIN       1536        /* smaller values encode 802.3 length */
 #define ETYPE_ARP       0x0806      /* Address Resolution packet    */
 #define ETYPE_IP        0x0800      /* Internet Protocol packet */
 #define ETYPE_IPV6      0x86DD      /* IPv6 over bluebook       */
@@ -48,6 +49,15 @@ typedef struct
     uint16_t vth_proto;  /* protocol field... */
 } VlanTagHdr;
 
+#define SNAP_SAPS       0xaaaa
+typedef struct
+{
+    uint16_t sh_dsap_ssap;
+    uint16_t sh_ctl_oc0;
+    uint16_t sh_oc1_oc2;
+    uint16_t sh_proto;
+} SnapHdr;
+
 typedef struct
 {
     DAQ_PktDecodeData_t decoded_data;
@@ -484,15 +494,21 @@ static inline bool decode_ip(const uint8_t *cursor, uint32_t len, DecodeData *dd
     dd->decoded_data.flags.bits.l3 = true;
     dd->decoded_data.flags.bits.ipv4 = true;
 
+    const uint16_t ipoff = ntohs(ip->frag_off);
+    const bool is_fragmented = (ipoff & IP_MF) != 0 || (ipoff & IP_OFFMASK) != 0;
     uint16_t offset = hlen;
-    switch (dd->ip->protocol)
+    
+    if (!is_fragmented) 
     {
-        case IPPROTO_TCP:
-            return decode_tcp(cursor + offset, len - offset, dd);
-        case IPPROTO_UDP:
-            return decode_udp(cursor + offset, len - offset, dd);
-        case IPPROTO_ICMP:
-            return decode_icmp(cursor + offset, len - offset, dd);
+        switch (dd->ip->protocol)
+        {
+            case IPPROTO_TCP:
+                return decode_tcp(cursor + offset, len - offset, dd);
+            case IPPROTO_UDP:
+                return decode_udp(cursor + offset, len - offset, dd);
+            case IPPROTO_ICMP:
+                return decode_icmp(cursor + offset, len - offset, dd);
+        }
     }
 
     cursor += offset;
@@ -526,6 +542,23 @@ static inline bool is_vlan_ethertype(uint16_t ether_type)
     }
 }
 
+static inline bool decode_snap(const uint8_t *cursor, uint32_t len, uint16_t *ether_type, uint16_t *offset)
+{
+    if (*ether_type >= ETYPE_MIN)
+        return true;
+
+    if (len < *offset + sizeof(SnapHdr))
+        return false;
+
+    const SnapHdr *snap = (const SnapHdr *) (cursor + *offset);
+    if (snap->sh_dsap_ssap != htons(SNAP_SAPS))
+        return false;
+
+    *ether_type = ntohs(snap->sh_proto);
+    *offset += sizeof(*snap);
+    return true;
+}
+
 static inline bool decode_eth(const uint8_t *cursor, uint32_t len, DecodeData *dd)
 {
     update_pyld_csum_offsets(cursor, dd);
@@ -538,6 +571,9 @@ static inline bool decode_eth(const uint8_t *cursor, uint32_t len, DecodeData *d
     uint16_t ether_type = ntohs(eth->ether_type);
     uint16_t offset = sizeof(*eth);
 
+    if (!decode_snap(cursor, len, &ether_type, &offset))
+        return false;
+
     dd->eth = eth;
     dd->decoded_data.flags.bits.l2 = true;
     dd->decoded_data.flags.bits.ethernet = true;
@@ -557,6 +593,9 @@ static inline bool decode_eth(const uint8_t *cursor, uint32_t len, DecodeData *d
             dd->decoded_data.flags.bits.vlan_qinq = true;
     }
 
+    if (!decode_snap(cursor, len, &ether_type, &offset))
+        return false;
+
     switch (ether_type)
     {
         case ETYPE_ARP:
